#!/bin/bash
#
# update.sh - Appliance Update script
#
# Type  : Utility
#
# Author: Rob Allen
#
# Date  : 12.01.2009
#
# Program Description
# ===================
# Script to manage and control update process
#
#
# Amendment History
# =================
# 12.01.09 Created today (RA)
# 09.07.09 Changed patch_count function, and invoke it directing output to null
# 05.02.10 Changed del patch command to remove directories.
#          

inv=`tput smso`
nor=`tput rmso`
UPLOAD=/backups/upload
PATCHBASE=/backups/patch
LOCK=/var/lock/subsys
TOMCAT=/usr/local/tomcat/webapps
TOMCAT_CONF=$TOMCAT/pinsafe/WEB-INF/conf
PINSAFE_TYPE="$1"
LocalhostIP="`grep Primary_IP /etc/mon/mon.cf|awk '{ print $3}'`"

initialise() {
 if test ! -d $UPLOAD; then
  mkdir $DIR; fi
 if test ! -d $PATCHBASE; then
  mkdir $PATCHBASE; fi
 if test ! -d $backups/config; then
  mkdir -p /backups/config; fi
}

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) 2010${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) 2010${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo 
 echo $1
 echo
}

s_heading() {
 clear
 echo "${inv} Swivel Maintenance (c) 2010${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) 2010${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo
}

menu() {
 heading "Patch/Update management"

 if test -f $PATCHBASE/README; then
  sub_menu patch; fi  

 if `ls $UPLOAD/patch.* > /dev/null 2>&1` && test -f $UPLOAD/pinsafe.war && test -f $UPLOAD/config.xml; then
  echo "1. Apply new Patch file"
  echo "2. Install new pinsafe.war file"
  echo "3. Install new config.xml file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu patch; menu; ;;
   2) sub_menu pinsafe; menu; ;;
   3) sub_menu config; menu; ;;
   *) exit; ;;
  esac
 fi

 if `ls $UPLOAD/patch.* > /dev/null 2>&1` && test -f $UPLOAD/pinsafe.war; then
  echo "1. Apply new Patch file"
  echo "2. Install new pinsafe.war file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu patch; menu; ;;
   2) sub_menu pinsafe; menu; ;;
   *) exit; ;;
  esac
 fi

 if `ls $UPLOAD/patch.* > /dev/null 2>&1` && test -f $UPLOAD/config.xml; then
  echo "1. Apply new Patch file"
  echo "2. Install new config.xml file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu patch; menu; ;;
   2) sub_menu config; menu; ;;
   *) exit; ;;
  esac
 fi

 if test -f $UPLOAD/pinsafe.war && test -f $UPLOAD/config.xml; then
  echo "1. Install new pinsafe.war file"
  echo "2. Install new config.xml file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu pinsafe; menu; ;;
   2) sub_menu config; menu; ;;
   *) exit; ;;
  esac
 fi

 if `ls $UPLOAD/patch.* > /dev/null 2>&1`; then
  echo "1. Install new patch file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu patch; menu; ;;
   *) exit; ;;
  esac
 fi

 if test -f $UPLOAD/pinsafe.war; then
  echo "1. Install new pinsafe.war file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu pinsafe; menu; ;;
   *) exit; ;;
  esac
 fi

 if test -f $UPLOAD/config.xml; then
  echo "1. Install new config.xml file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu config; menu; ;;
   *) exit; ;;
  esac
 fi
}

sub_menu() {
 heading "Patch/Update management"

 if test $1 = "patch"; then
  if test -f $PATCHBASE/README; then
   echo "1. Display Patch information"
   echo "2. Apply Patch"
   echo "3. Remove patch temporary files"
   echo "0. Exit"
   echo
   read -p "Select: " option
   case $option in
    1) disp_patch_readme; sub_menu patch; ;;
    2) apply_patch; sub_menu patch; ;;
    3) remove_patch; sub_menu patch; ;;
    *) exit; ;;
   esac
  fi

  if `ls $UPLOAD/patch.* > /dev/null 2>&1`; then
   patch_count > /dev/null 2>&1
   if test $patch_qty != 1; then
    echo "Only one patch file may be uploaded at a time"
    echo; read -p "Press Return to Continue" yn
    exit; fi
   patch_name=`ls $UPLOAD/patch.*`
   patch_name=`basename $patch_name`
   echo "1. Import patch file: $patch_name"
   echo "2. Delete patch file: $patch_name"
   echo "0. Exit"
   echo
   read -p "Select: " option
   case $option in
    1) import_patch; menu; ;;
    2) del_file $patch_name ; menu; ;;
    *) menu; ;;
   esac
  fi
 fi

 if test $1 = "config"; then
  if test -f $UPLOAD/config.xml; then
   echo "1. Upgrade config.xml"
   echo "2. Delete config.xml"
   echo "0. Exit"
   echo
   read -p "Select: " option
   case $option in
    1) upgrade_PIN_config; menu; ;;
    2) del_file config.xml; menu; ;;
    *) menu; ;;
   esac
  fi
 fi

 if test $1 = "pinsafe"; then
  if test -f $UPLOAD/pinsafe.war; then
   echo "1. Deploy pinsafe.war file"
   echo "2. Delete pinsafe.war file"
   echo "0. Exit"
   echo
   read -p "Select: " option
   case $option in
    1) deploy_pinsafe_menu; menu; ;;
    2) del_file pinsafe.war; menu; ;;
    *) menu; ;;
   esac
  fi
 fi
 menu
}

del_file() {
 heading "Delete file"
 echo "Delete $1"
 yorn menu
 rm -f $UPLOAD/$1
 echo; echo "$1 Deleted"
 sleep 2
}

patch_count() {
 patch_qty=`ls $UPLOAD/patch.*|wc -l`
}

deploy_pinsafe_menu() {
 heading "Deploy PINsafe"
 chk_tomcat menu
 if test -d $TOMCAT/pinsafe; then
  echo "1. Install alongside existing PINsafe installation"
  echo "2. Move existing installation to backup area (still accessible)"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) deploy_pinsafe alongside; deploy_pinsafe_menu; ;;
   2) deploy_pinsafe replace; deploy_pinsafe_menu; ;;
   *) sub_menu pinsafe; ;;
  esac
 else
  deploy_pinsafe new; deploy_pinsafe_menu
 fi
}

deploy_pinsafe() {
 heading "Deploy PINsafe"
 if test $1 = "alongside"; then
  if test -d $TOMCAT/pinsafe-n; then
   echo "There is already a PINsafe-n deployment."
   echo
   echo "Remove existing pinsafe-n deployment and pinsafe-n.war file?"
   yorn deploy_pinsafe_menu
   rm -rf $TOMCAT/pinsafe-n.war $TOMCAT/pinsafe-n; fi

  heading "Deploy PINsafe"
  echo "Deploying a new version of PINsafe. However the existing installation will"
  echo "continue to operate without any interruption."
  echo; echo "Deploying new pinsafe.war file as ${inv} pinsafe-n ${nor}"
  yorn deploy_pinsafe_menu
  mv $UPLOAD/pinsafe.war $TOMCAT/pinsafe-n.war
  echo "pinsafe.war file moved"
  service tomcat5 start >/dev/null 2>&1
  echo "Deploying pinsafe archive, this may take a few minutes"
  service tomcat5 stop >/dev/null 2>&1
  heading "PINsafe Deployed"
  echo "Access PINsafe Deployed access on: https://$LocalhostIP:8080/pinsafe-n"
  migrate_settings pinsafe pinsafe-n
 fi

 if test $1 = "replace"; then
  if test -d $TOMCAT/pinsafe-o; then
   echo "There is already a PINsafe-o deployment."
   echo
   echo "Remove existing pinsafe-o deployment and pinsafe-o.war file?"
   yorn deploy_pinsafe_menu
   rm -rf $TOMCAT/pinsafe-o.war $TOMCAT/pinsafe-o; fi

  heading "Deploy PINsafe"
  echo "Deploying a new version of PINsafe. The existing installation will be"
  echo "renamed to ${inv} pinsafe-o ${nor}"
  echo; echo "Deploying new pinsafe.war file as ${inv} pinsafe ${nor}"
  yorn deploy_pinsafe_menu
  mv $TOMCAT/pinsafe.war $TOMCAT/pinsafe-o.war
  mv $TOMCAT/pinsafe $TOMCAT/pinsafe-o
  mv $UPLOAD/pinsafe.war $TOMCAT/pinsafe.war
  echo "pinsafe.war file moved"
  service tomcat5 start >/dev/null 2>&1
  echo "Deploying pinsafe archive, this may take a few minutes"
  service tomcat5 stop >/dev/null 2>&1
  echo "Access PINsafe Deployed access on: https://$LocalhostIP:8080/pinsafe"
  migrate_settings pinsafe-o pinsafe
 fi

 if test $1 = "new"; then
  echo "Deploying a new version of PINsafe."
  echo; echo "Deploying new pinsafe.war file as ${inv} pinsafe ${nor}"
  yorn menu
  mv $UPLOAD/pinsafe.war $TOMCAT/pinsafe.war
  echo "pinsafe.war file moved"
  service tomcat5 start >/dev/null 2>&1
  echo "Deploying pinsafe archive, this may take a few minutes"
  service tomcat5 stop >/dev/null 2>&1
  heading "PINsafe Deployed"
  echo "Access PINsafe Deployed access on: https://$LocalhostIP:8080/pinsafe"
  echo
 fi

 read -p "Press Return to Continue" yn
 exit
}

migrate_settings() {
 heading "PINsafe Deployed"
 echo "1. Migrate settings from original deployment?"
 echo "0. Exit"
 echo
 read -p "Select: " option
 case $option in
  1) echo; echo "Watch this space..." yn; ;;
  0) exit; ;;
  *) migrate_settings; ;;
 esac

 cp -pf $TOMCAT/$1/WEB-INF/conf/config.xml $TOMCAT/$2/WEB-INF/conf/config.xml
 cp -pf $TOMCAT/$1/WEB-INF/conf/ranges.xml $TOMCAT/$2/WEB-INF/conf/ranges.xml
 cp -pf $TOMCAT/$1/WEB-INF/data/repository.xml $TOMCAT/$2/WEB-INF/data/repository.xml
 cp -pfr $TOMCAT/$1/WEB-INF/db $TOMCAT/$2/WEB-INF
}

upgrade_PIN_config() {
 heading "Upgrade PINsafe configuration"
 chk_tomcat menu
 echo "By continuing the PINsafe config.xml will be replaced by the config.xml"
 echo "file held in the \"upload\" directory."
 yorn menu
 if test -f $TOMCAT_CONF/config.xml; then
  cp $TOMCAT_CONF/config.xml /backups/config/config_$$_`date +%d%m%y`.xml
  mv -f /backups/upload/config.xml $TOMCAT_CONF
  echo; echo -n "PINsafe configuration updated"
 else
  echo; echo -n "PINsafe may not be deployed, as the destination directory is not available"
 fi
 sleep 3
}

chk_tomcat() {
 if test -f $LOCK/tomcat5; then
  echo "Tomcat is running and must be stopped before continuing"
  echo; echo -n "Press Return to Continue"
  read yn
  $1; fi
}

get_patch_ver() {
 PATCH_VER=`cat $PATCHBASE/patch_ver`
}

apply_patch() {
 heading "Apply Patch/Update"
 get_patch_ver
 echo "Continuing will apply patch ${inv}$PATCH_VER${nor}. It is suggested that the system is not"
 echo "authenticating users whilst the patch is applied."
 yorn menu
 sh $PATCHBASE/install.sh 
}

remove_patch() {
 heading "Patch/Update management"
 echo "Removing temporary patch files. Once the files have been removed the patch will"
 echo "require uploading again using SFTP."
 yorn menu
 rm -rf $PATCHBASE/*
}

disp_patch_readme() {
 s_heading
 get_patch_ver
 echo "Patch Version: $PATCH_VER"
 echo
 more $PATCHBASE/README
 echo; read -p "Press Return to Continue" yn
}

import_patch() {
 s_heading
 patch_count > /dev/null 2>&1
 if test $patch_qty = 1; then
  patch_name=`ls $UPLOAD/patch.*`
  patch_name=`basename $patch_name`
  echo "An update file has been placed in the upload area."
  echo
  echo "Please wait whilst the patch file is checked"
  sleep 2
  process_patch
 else
  echo -n "Only one patch file may be uploaded at a time"
  sleep 2
 fi
}

process_patch() {
 mv $UPLOAD/$patch_name $PATCHBASE
 cd $PATCHBASE
# gpg $patch_name >> /var/log/cmi/patch.log 2>&1
 count=`echo $patch_name | wc -c`; count=`expr $count - 1`
 patch_name=`echo $patch_name|cut -c -$count`
 echo

 if test -f $patch_name; then
  echo "Patch file valid and now available."
  tar xfz $PATCHBASE/$patch_name
 else
  echo "The patch file is invalid."
 # mv $patch_name.gpg $UPLOAD/failed_$patch_name.gpg
 fi

 read -p "Press Return to Continue"
}

yorn() {
 echo
 read -p "Are you sure? (Yes/No): " yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y" ; then
  $1; fi
}

initialise
menu
