#!/bin/bash 
# 
# Written by Raymond Butler 
# Swivel Secure Ltd (c)2011
# v0.9 07/02/2010 
# cmi_0.9z-17-rc2
# Script to update PINsafe Appliance with CMI with some fixes. 
# 
# 6.12.10 Various fixes in the 'cleanup_backup' function. (RA) 
#

APPLIANCE_UPGRADE_VER="2.0.12-r2"
FILE_DATE=`date "+%H%M_%d-%b-%Y"`
BACKUP_FILENAME=".backup_existing_cmi-$FILE_DATE.tgz"
MYSQL_FILENAME="mysql_full-$FILE_DATE.SQL"
PINSAFE_CONF="/etc/pinsafe.conf"
PINSAFE_CONF_NEW="/tmp/pinsafe.conf.new"
CMI_UPDATE_FILE="cmi_update_0.9z-17_rc2.tgz"
NET_SCRIPTS="/etc/sysconfig/network-scripts"
LOCK="/var/lock/subsys"
LOGFILE="./logfile-$FILE_DATE.log"
ERROR_LOGFILE="./error-$FILE_DATE.log"
PATCHBASE=/backups/patch

# echo "Logging to: $LOGFILE"

>$PINSAFE_CONF_NEW

if test -f /etc/init.d/mysqld; then
    MYSQL=mysqld
else
    MYSQL=mysql
fi

cleanup() {
 cd $PATCHBASE
 rm -rf etc bin rpms usr updates 
 rm -rf $CMI_UPDATE_FILE $PINSAFE_CONF_NEW update_now.sh
 rm -f /tmp/$MYSQL_FILENAME
 cd $PATCHBASE; rm -rf *
 if test -f /tmp/$BACKUP_FILENAME; then
  mv -f /tmp/$BACKUP_FILENAME /backups; fi
 rm -rf /backups/.default
 echo; echo "Clean up done..."
}

cleanup_backup() {
 cd /backups
 rm -f purge*
 for i in *.log; do
  if test -f $i; then
   dname=`echo $i|cut -d\_ -f1`
   if test -d $dname; then
    date=`cat $dname/backup.date`
    type=`echo $i|cut -d\_ -f2|cut -d\. -f1`
   else
    dname=""
   fi

   if test $dname && test $date && test $type; then
    if test -f $dname/backup_comment.txt; then
     comment=`cat $dname/backup_comment.txt`; fi

    if test $type = "all"; then
     dtype="Full Backup"
    elif test $type = "pin"; then
     dtype="PINsafe Backup"; fi

    echo "DATE=$date" > $dname.info
    echo "TYPE=$dtype" >> $dname.info
    echo "DB_IN_USE=$DB_IN_USE" >> $BACKUPROOT/$dir_name.info
    if test -f $dname/backup_comment.txt; then
     echo "COMMENT=$comment" >> $dname.info; fi
    echo "DB_IN_USE=$DB_IN_USE" >> $BACKUPROOT/$dir_name.info

    if test $type = "all"; then
     fname="`echo $dname`_full.tar.gz"
     mv $fname $dname.tar.gz; fi

    if test $type = "pin"; then
     fname="`echo $dname`_pin.tar.gz"
     mv $fname $dname.tar.gz; fi

    rm -rf $dname $i latest
   else
    echo "A value is missing from $i, unable to migrate backup file"
   fi
  fi
 done
 echo; echo "Clean up backups done..."
}

scan_orphaned() { 
 echo; echo "Scanning for orphaned backup directories"
 echo
 
 echo -n "Scanning: "
 rm -f /tmp/orphaned_dir.txt
  
 cd /backups
 for file in `ls -l |grep ^d | awk '{print $9}'|grep ^[0-9]`; do
  echo -n "." 
  if test ! -f `echo $file`_*.tar.gz; then
   echo $file >> /tmp/orphaned_dir.txt
  fi
 done
  
 if test -f /tmp/orphaned_dir.txt; then
  echo
  echo "There are orphaned directories."
  echo "Deleting orphaned directories" 
  rm -rf `cat /tmp/orphaned_dir.txt`
  echo "orphaned directories removed"
  rm -f /tmp/orphaned_dir.txt
 else
  echo; echo "No directories matching criteria"
 fi
}

check_running() {
 if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe" 2>/dev/null`"; then
  if test ! -f $LOCK/$1; then
   touch $LOCK/$1; fi
 else
  if test -f $LOCK/$1; then
   rm -f $LOCK/$1; fi
 fi

 if test -f $LOCK/$1; then
  value="Running"
 else
  value="Stopped"
 fi
}

get_value() {
 VALUE=`grep ^$1= $PINSAFE_CONF | cut -d\= -f2`
 if test $VALUE; then
  echo "$1=$VALUE" >> $PINSAFE_CONF_NEW
 else
  echo "$1=$2" >> $PINSAFE_CONF_NEW
 fi
}

check_drbd() {
 check_running drbd drbd
 if test $value = "Running"; then
  get_value DRBD yes
 else
  get_value DRBD no
 fi
}

set_values() {
 OLD_APPLIANCE_VERS=`grep ^APPLIANCE_VERS= $PINSAFE_CONF | cut -d\= -f2`
 echo "APPLIANCE_VERS=$APPLIANCE_UPGRADE_VER" >> $PINSAFE_CONF_NEW
 get_value archives 30
 check_drbd
 get_value APPLIANCE_PATCH_VER p2.0.1
 get_value PINSAFE_PATCH_VER p2.0.1
 get_value logs 30
 get_value PINSAFE_TYPE Single 
 get_value DR no
 get_value ActiveIP 0.0.0.0
 get_value REP_INTERF eth1
 get_value NTP_SERVER
 get_value SMTP
}

get_ftp() {
 grep ^FTP_SERVER $PINSAFE_CONF >> $PINSAFE_CONF_NEW
 grep ^FTP_USERNAME $PINSAFE_CONF >> $PINSAFE_CONF_NEW
 grep ^FTP_PASSWORD $PINSAFE_CONF >> $PINSAFE_CONF_NEW
 grep ^FTP_DEST $PINSAFE_CONF >> $PINSAFE_CONF_NEW
}

yorn() {
 echo
 read -p "Are you sure? (Yes/No): " yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y"; then
  $1; fi
}

quit() {
 echo "Patch NOT applied."
 echo "Exiting..."
 exit
}

db_used() {
 CONFIG_XML=/usr/local/tomcat/webapps/pinsafe/WEB-INF/conf/config.xml
 XML="<option generated=\"true\" selected=\"true\""

 DB_IN_USE=Internal
 if test -f $CONFIG_XML; then
  DB_IN_USE=Shipping
  if grep "$XML>Shipping</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Shipping; fi
  if grep "$XML>Internal</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Internal; fi
  if grep "$XML>MySQL 5</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=MySQL; fi
 else
  if test ! $APPLIANCE_TYPE = "AP"; then
   echo "Unable to find the config.xml file."; fi
 fi
}

db_used

if test $DB_IN_USE = "MySQL"; then
 check_running $MYSQL mysqld
 if test $value = "Running"; then
  MYSQL_RUNNING="YES"
 else
  echo "MYSQL not running. Start MySQL before continuing."
  echo; read -p "Press any key to exit" yn
  quit
 fi
fi

echo; echo "Preparing to update CMI."
yorn quit

echo "Creating backup file: /tmp/$BACKUP_FILENAME"
echo

if test $MYSQL_RUNNING; then
 if test $MYSQL_RUNNING = "YES"; then
  mysqldump --single-transaction --flush-logs --all-databases > /tmp/$MYSQL_FILENAME
 fi
else
 echo "MySQL not used. DB in use: $DB_IN_USE" > /tmp/$MYSQL_FILENAME
fi

tar czf /tmp/$BACKUP_FILENAME /etc/pinsafe.conf /etc/pinsafe_motd /etc/issue /usr/local/etc /usr/local/bin /etc/cron.daily/*.sh /etc/logrotate.d /etc/profile /home/admin/.* /home/swivel/.* /etc/webmin /usr/libexec/webmin /tmp/$MYSQL_FILENAME 2>/dev/null

echo "Backup Completed"
echo

# Upgrade PINsafe.conf file
set_values
get_ftp
cp -f $PINSAFE_CONF_NEW /etc/pinsafe.conf

if test -f /usr/local/bin/tw_cli; then
 mv /usr/local/bin/tw_cli /tmp; fi

if test -f /usr/local/bin/email; then
 mv /usr/local/bin/email /tmp; fi

rm -f /usr/local/bin/*

if test -f /tmp/tw_cli; then
 mv /tmp/tw_cli /usr/local/bin; fi

if test -f /tmp/email; then
 mv /tmp/email /usr/local/bin; fi

cp etc/cron.daily/* /etc/cron.daily
cp etc/logrotate.d/* /etc/logrotate.d
cp bin/* /usr/local/bin

sed -i 's/\/usr\/local\/bin\/smenu.sh/\/usr\/local\/bin\/smenu/g' /home/admin/.bash_profile
sed -i '/export PINSAFE_TYPE/d' /etc/profile
sed -i '/export APPLIANCE_VER/d' /etc/profile
sed -i "/profile/ a\trap '' 2" /etc/profile
sed -i '/profile/ a\trap - INT TERM EXIT' /etc/profile

echo "Updating RPM Signing Keys"                                                                                          
rpm -import rpms/RPM-GPG-KEY-RedHat-4                                                                                     
rpm -import rpms/RPM-GPG-KEY-CentOS-4                                                                                     
rpm -import rpms/RPM-GPG-KEY-RedHat-5                                                                                     
rpm -import rpms/RPM-GPG-KEY-CentOS-5                                                                                     
rpm -import rpms/RPM-GPG-KEY.mysql                                                                                        
rpm -import rpms/RPM-GPG-KEY.dag                                                                                          
rpm -import rpms/RPM-GPG-KEY.opensuse-11.1                                                                                
rpm -import rpms/webmin-1.480-1.noarch.rpm.key

echo; echo "Installing / Updating required packages."
rpm  -Uh rpms/email-3.1.2-1.RHEL4.i386.rpm
rpm  -Uh rpms/MegaCli-2.00.12-1.i386.rpm
rpm  -Uh rpms/nmap-3.70-1.i386.rpm
rpm  --nosignature -Uh rpms/mpt-status-1.2.0-51.4.i586.rpm

echo; echo "Updating WEBMIN"
service webmin stop
rpm -ev webmin
rm -rf /etc/webmin.rpmsave
rm -rf /usr/libexec/webmin

# START Webmin Install
rpm -ih rpms/webmin-1.510-1.noarch.rpm
service webmin stop
rm -f /etc/webmin/module.infos.cache
cd /
tar zxf $PATCHBASE/updates/webmin-1.510-1_pinsafe_install.tgz
service webmin start
echo "webmin update complete."

cd $PATCHBASE
echo

PINSAFE_TYPE=`grep PINSAFE_TYPE $PINSAFE_CONF|cut -d\= -f2`
if test "$PINSAFE_TYPE" = "Single"; then
 PINSAFE_TYPE="Single "; fi

if test "$PINSAFE_TYPE" = "Single "; then
 echo "Updating Single"
 echo "#"
 rm -f /etc/logrotate.d/heartbeat
 sed -i "s/^REP_INTERf=eth1/REP_INTERF=eth0/g" /etc/pinsafe.conf
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/pinsafe_motd
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/issue
 echo "##"
 echo "Updated Single."
fi

if test "$PINSAFE_TYPE" = "DR"; then
 echo "Updating DR"
 echo "#"
 rm -f /etc/logrotate.d/heartbeat
 sed -i "s/^REP_INTERf=eth1/REP_INTERF=eth0/g" /etc/pinsafe.conf
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/pinsafe_motd
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/issue
 echo "##"
 echo "Updated DR."
fi

if test "$PINSAFE_TYPE" = "Primary"; then
 echo "Updating Primary"
 echo "#"
 sed -i 's/\/var\/log\/heartbeat.log/\/var\/log\/ha-log/' /etc/ha.d/ha.cf
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/pinsafe_motd
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/issue
 echo "##"
 echo "Updated Primary."
fi

if test "$PINSAFE_TYPE" = "Standby"; then
 echo "Updating Standby"
 echo "#"
 sed -i 's/\/var\/log\/heartbeat.log/\/var\/log\/ha-log/' /etc/ha.d/ha.cf
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/pinsafe_motd
# sed -i "s/$OLD_APPLIANCE_VERS/$APPLIANCE_UPGRADE_VER/g" /etc/issue
 echo "##"
 cp usr/lib/mon/alert.d/ha-standby.alert /usr/lib/mon/alert.d
 echo "###"
 echo "Updated Standby."
fi

echo

if test $MYSQL_RUNNING; then
 if test "$MYSQL_RUNNING" = "YES"; then
  echo "mysql permission updates applied."
  mysql -uroot -e"GRANT ALL PRIVILEGES ON *.* TO 'admin'@'%' IDENTIFIED BY 'lockbox';"
  mysql -uroot -e"GRANT ALL PRIVILEGES ON pinsafe.* TO 'pinsafe'@localhost;"
  mysql -uroot -e"GRANT ALL PRIVILEGES ON pinsafe_rep.* TO 'pinsafe'@localhost;"
  mysql -uroot -e"FLUSH PRIVILEGES;"
 fi
fi

cleanup
cleanup_backup
scan_orphaned

echo "Patch applied. Log out to apply changes."
